@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2021 Genome Research Ltd.
+ * Copyright (c) 2017-2023 Genome Research Ltd.
  * Author(s): James Bonfield
  *
  * Redistribution and use in source and binary forms, with or without
@@ -104,7 +104,8 @@ unsigned char *rans_compress_O0_32x16_neon(unsigned char *in,
         goto empty;
 
     // Compute statistics
-    hist8(in, in_size, F);
+    if (hist8(in, in_size, F) < 0)
+        return NULL;
 
     // Normalise so frequences sum to power of 2
     uint32_t fsum = in_size;
@@ -1444,7 +1445,7 @@ unsigned char *rans_uncompress_O1_32x16_neon(unsigned char *in,
         uint32_t u_freq_sz, c_freq_sz;
         cp += var_get_u32(cp, cp_end, &u_freq_sz);
         cp += var_get_u32(cp, cp_end, &c_freq_sz);
-        if (c_freq_sz >= cp_end - cp - 16)
+        if (c_freq_sz > cp_end - cp)
             goto err;
         tab_end = cp + c_freq_sz;
         if (!(c_freq = rans_uncompress_O0_4x16(cp, c_freq_sz, NULL, u_freq_sz)))
@@ -1955,4 +1956,7 @@ unsigned char *rans_uncompress_O1_32x16_neon(unsigned char *in,
 }
 
 #undef MAGIC2
+#else  /* __ARM_NEON */
+// Prevent "empty translation unit" errors when building without NEON
+const char *rANS_static32x16pr_neon_disabled = "No NEON";
 #endif /* __ARM_NEON */
